@@ -1,4 +1,4 @@
-/* $Id: browser.c 5579 2016-01-21 20:44:22Z bens $ */
+/* $Id: browser.c 5677 2016-02-25 14:08:47Z bens $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
@@ -61,6 +61,7 @@ char *do_browser(char *path, DIR *dir)
     functionptrtype func;
 	/* The function of the key the user typed in. */
 
+    /* Don't show a cursor in the file list. */
     curs_set(0);
     blank_statusbar();
     bottombars(MBROWSER);
@@ -115,9 +116,23 @@ char *do_browser(char *path, DIR *dir)
 		/* The path we switch to at the "Go to Directory"
 		 * prompt. */
 
-	/* Display the file list if we don't have a key, or if the
-	 * selected file has changed, and set width in the process. */
-	if (kbinput == ERR || old_selected != selected)
+	/* Make sure that the cursor is off. */
+	curs_set(0);
+
+#ifndef NANO_TINY
+	if (kbinput == KEY_WINCH) {
+	    /* Rebuild the file list and sort it. */
+	    browser_init(path_save, opendir(path_save));
+	    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
+
+	    /* Make sure the selected file is within range. */
+	    if (selected >= filelist_len)
+		selected = filelist_len - 1;
+	}
+#endif
+	/* Display (or redisplay) the file list if we don't have a key yet,
+	 * or the list has changed, or the selected file has changed. */
+	if (kbinput == ERR || kbinput == KEY_WINCH || old_selected != selected)
 	    browser_refresh();
 
 	old_selected = selected;
@@ -125,11 +140,8 @@ char *do_browser(char *path, DIR *dir)
 	kbinput = get_kbinput(edit);
 
 #ifndef NANO_TINY
-	if (kbinput == KEY_WINCH) {
-	    kbinput = ERR;
-	    curs_set(0);
+	if (kbinput == KEY_WINCH)
 	    continue;
-	}
 #endif
 
 #ifndef DISABLE_MOUSE
@@ -170,17 +182,14 @@ char *do_browser(char *path, DIR *dir)
 	} else if (func == do_help_void) {
 #ifndef DISABLE_HELP
 	    do_help_void();
-	    /* Perhaps the window dimensions have changed. */
-	    browser_refresh();
-	    curs_set(0);
+	    /* The window dimensions might have changed, so act as if. */
+	    kbinput = KEY_WINCH;
 #else
 	    say_there_is_no_help();
 #endif
 	} else if (func == do_search) {
 	    /* Search for a filename. */
-	    curs_set(1);
 	    do_filesearch();
-	    curs_set(0);
 	} else if (func == do_research) {
 	    /* Search for another filename. */
 	    do_fileresearch();
@@ -199,7 +208,6 @@ char *do_browser(char *path, DIR *dir)
 	    selected = filelist_len - 1;
 	} else if (func == goto_dir_void) {
 	    /* Go to a specific directory. */
-	    curs_set(1);
 	    i = do_prompt(TRUE,
 #ifndef DISABLE_TABCOMP
 			FALSE,
@@ -211,7 +219,6 @@ char *do_browser(char *path, DIR *dir)
 			/* TRANSLATORS: This is a prompt. */
 			browser_refresh, _("Go To Directory"));
 
-	    curs_set(0);
 	    bottombars(MBROWSER);
 
 	    /* If the directory begins with a newline (i.e. an
@@ -311,22 +318,22 @@ char *do_browser(char *path, DIR *dir)
 		 * get out. */
 		retval = mallocstrcpy(NULL, filelist[selected]);
 		break;
-	    } else if (strcmp(tail(filelist[selected]), "..") == 0)
-		/* We've successfully opened the parent directory,
-		 * save the current directory in prev_dir, so that
-		 * we can easily return to it by hitting Enter. */
-		prev_dir = mallocstrcpy(NULL, striponedir(filelist[selected]));
+	    }
 
 	    dir = opendir(filelist[selected]);
+
 	    if (dir == NULL) {
-		/* We can't open this directory for some reason.
-		 * Complain. */
 		statusbar(_("Error reading %s: %s"),
 				filelist[selected], strerror(errno));
 		beep();
 		continue;
 	    }
 
+	    /* If we moved up one level, remember where we came from, so
+	     * this directory can be highlighted and easily reentered. */
+	    if (strcmp(tail(filelist[selected]), "..") == 0)
+		prev_dir = striponedir(filelist[selected]);
+
 	    path = mallocstrcpy(path, filelist[selected]);
 
 	    /* Start over again with the new path value. */
@@ -338,7 +345,6 @@ char *do_browser(char *path, DIR *dir)
     }
     titlebar(NULL);
     edit_refresh();
-    curs_set(1);
     if (old_const_update)
 	SET(CONST_UPDATE);
 
@@ -548,14 +554,6 @@ void browser_refresh(void)
     char *foo;
 	/* The additional information that we'll display about a file. */
 
-    /* Perhaps window dimensions have changed; reinitialize the browser. */
-    browser_init(path_save, opendir(path_save));
-    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
-
-    /* Make sure the selected file is within range. */
-    if (selected >= filelist_len)
-	selected = filelist_len - 1;
-
     titlebar(path_save);
     blank_edit();
 
