@@ -1,4 +1,4 @@
-/* $Id: color.c 5477 2015-12-04 20:54:34Z bens $ */
+/* $Id: color.c 5538 2016-01-09 18:41:56Z bens $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 #include <unistd.h>
 
 #ifdef HAVE_MAGIC_H
@@ -34,31 +35,34 @@
 
 #ifndef DISABLE_COLOR
 
-/* For each syntax list entry, go through the list of colors and assign
- * the color pairs. */
+/* Initialize the colors for nano's interface, and assign pair numbers
+ * for the colors in each syntax. */
 void set_colorpairs(void)
 {
     const syntaxtype *this_syntax = syntaxes;
-    bool defok = FALSE;
-    short fg, bg;
+    bool using_defaults = FALSE;
+    short foreground, background;
     size_t i;
 
+    /* Tell ncurses to enable colors. */
     start_color();
 
 #ifdef HAVE_USE_DEFAULT_COLORS
-    /* Use the default colors, if available. */
-    defok = (use_default_colors() != ERR);
+    /* Allow using the default colors, if available. */
+    using_defaults = (use_default_colors() != ERR);
 #endif
 
+    /* Initialize the color pairs for nano's interface elements. */
     for (i = 0; i < NUMBER_OF_ELEMENTS; i++) {
 	bool bright = FALSE;
 
-	if (parse_color_names(specified_color_combo[i], &fg, &bg, &bright)) {
-	    if (fg == -1 && !defok)
-		fg = COLOR_WHITE;
-	    if (bg == -1 && !defok)
-		bg = COLOR_BLACK;
-	    init_pair(i + 1, fg, bg);
+	if (parse_color_names(specified_color_combo[i],
+			&foreground, &background, &bright)) {
+	    if (foreground == -1 && !using_defaults)
+		foreground = COLOR_WHITE;
+	    if (background == -1 && !using_defaults)
+		background = COLOR_BLACK;
+	    init_pair(i + 1, foreground, background);
 	    interface_color_pair[i].bright = bright;
 	    interface_color_pair[i].pairnum = COLOR_PAIR(i + 1);
 	}
@@ -74,6 +78,8 @@ void set_colorpairs(void)
 	specified_color_combo[i] = NULL;
     }
 
+    /* For each syntax, go through its list of colors and assign each
+     * its pair number, giving identical color pairs the same number. */
     for (; this_syntax != NULL; this_syntax = this_syntax->next) {
 	colortype *this_color = this_syntax->color;
 	int clr_pair = NUMBER_OF_ELEMENTS + 1;
@@ -81,12 +87,11 @@ void set_colorpairs(void)
 	for (; this_color != NULL; this_color = this_color->next) {
 	    const colortype *beforenow = this_syntax->color;
 
-	    for (; beforenow != this_color &&
-		(beforenow->fg != this_color->fg ||
-		beforenow->bg != this_color->bg ||
-		beforenow->bright != this_color->bright);
-		beforenow = beforenow->next)
-		;
+	    while (beforenow != this_color &&
+			(beforenow->fg != this_color->fg ||
+			beforenow->bg != this_color->bg ||
+			beforenow->bright != this_color->bright))
+		beforenow = beforenow->next;
 
 	    if (beforenow != this_color)
 		this_color->pairnum = beforenow->pairnum;
@@ -101,38 +106,36 @@ void set_colorpairs(void)
 /* Initialize the color information. */
 void color_init(void)
 {
+    colortype *tmpcolor = openfile->colorstrings;
+    bool using_defaults = FALSE;
+    short foreground, background;
+
     assert(openfile != NULL);
 
-    if (has_colors()) {
-	const colortype *tmpcolor;
+    /* If the terminal is not capable of colors, forget it. */
+    if (!has_colors())
+	return;
+
 #ifdef HAVE_USE_DEFAULT_COLORS
-	/* Use the default colors, if available. */
-	bool defok = (use_default_colors() != ERR);
+    /* Allow using the default colors, if available. */
+    using_defaults = (use_default_colors() != ERR);
 #endif
 
-	for (tmpcolor = openfile->colorstrings; tmpcolor != NULL;
-		tmpcolor = tmpcolor->next) {
-	    short foreground = tmpcolor->fg, background = tmpcolor->bg;
-	    if (foreground == -1) {
-#ifdef HAVE_USE_DEFAULT_COLORS
-		if (!defok)
-#endif
-		    foreground = COLOR_WHITE;
-	    }
+    /* For each coloring expression, initialize the color pair. */
+    for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
+	foreground = tmpcolor->fg;
+	background = tmpcolor->bg;
 
-	    if (background == -1) {
-#ifdef HAVE_USE_DEFAULT_COLORS
-		if (!defok)
-#endif
-		    background = COLOR_BLACK;
-	    }
+	if (foreground == -1 && !using_defaults)
+	    foreground = COLOR_WHITE;
 
-	    init_pair(tmpcolor->pairnum, foreground, background);
+	if (background == -1 && !using_defaults)
+	    background = COLOR_BLACK;
 
+	init_pair(tmpcolor->pairnum, foreground, background);
 #ifdef DEBUG
-	    fprintf(stderr, "init_pair(): fg = %hd, bg = %hd\n", tmpcolor->fg, tmpcolor->bg);
+	fprintf(stderr, "init_pair(): fg = %hd, bg = %hd\n", foreground, background);
 #endif
-	}
     }
 }
 
@@ -414,7 +417,8 @@ void reset_multis(filestruct *fileptr, bool force)
     regmatch_t startmatch, endmatch;
     const colortype *tmpcolor = openfile->colorstrings;
 
-    if (!openfile->syntax)
+    /* If there is no syntax or no multiline regex, there is nothing to do. */
+    if (openfile->syntax == NULL || openfile->syntax->nmultis == 0)
 	return;
 
     for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
@@ -448,4 +452,151 @@ void reset_multis(filestruct *fileptr, bool force)
     }
 }
 
+/* Allocate (for one line) the cache space for multiline color regexes. */
+void alloc_multidata_if_needed(filestruct *fileptr)
+{
+    int i;
+
+    if (fileptr->multidata == NULL) {
+	fileptr->multidata = (short *)nmalloc(openfile->syntax->nmultis * sizeof(short));
+
+	for (i = 0; i < openfile->syntax->nmultis; i++)
+	    fileptr->multidata[i] = -1;
+    }
+}
+
+/* Poll the keyboard every second to see if the user starts typing. */
+bool key_was_pressed(void)
+{
+    static time_t last_time = 0;
+
+    if (time(NULL) != last_time) {
+	last_time = time(NULL);
+	return (wgetch(edit) != ERR);
+    } else
+	return FALSE;
+}
+
+/* Precalculate the multi-line start and end regex info so we can
+ * speed up rendering (with any hope at all...). */
+void precalc_multicolorinfo(void)
+{
+    const colortype *tmpcolor = openfile->colorstrings;
+    regmatch_t startmatch, endmatch;
+    filestruct *fileptr, *endptr;
+
+    if (openfile->colorstrings == NULL || ISSET(NO_COLOR_SYNTAX))
+	return;
+
+#ifdef DEBUG
+    fprintf(stderr, "Entering precalculation of multiline color info\n");
+#endif
+    /* Let us get keypresses to see if the user is trying to start
+     * editing.  Later we may want to throw up a statusbar message
+     * before starting this if it takes too long to do this routine.
+     * For now silently abort if they hit a key. */
+    nodelay(edit, TRUE);
+
+    for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
+	/* If this is not a multi-line regex, skip it. */
+	if (tmpcolor->end == NULL)
+	    continue;
+#ifdef DEBUG
+	fprintf(stderr, "Starting work on color id %d\n", tmpcolor->id);
+#endif
+
+	for (fileptr = openfile->fileage; fileptr != NULL; fileptr = fileptr->next) {
+	    int startx = 0, nostart = 0;
+
+	    if (key_was_pressed())
+		goto precalc_cleanup;
+#ifdef DEBUG
+	    fprintf(stderr, "working on lineno %ld... ", (long)fileptr->lineno);
+#endif
+	    alloc_multidata_if_needed(fileptr);
+
+	    while ((nostart = regexec(tmpcolor->start, &fileptr->data[startx],
+			1, &startmatch, (startx == 0) ? 0 : REG_NOTBOL)) == 0) {
+		/* Look for an end, and start marking how many lines are
+		 * encompassed, which should speed up rendering later. */
+		startx += startmatch.rm_eo;
+#ifdef DEBUG
+		fprintf(stderr, "start found at pos %lu... ", (unsigned long)startx);
+#endif
+		/* Look first on this line for an end. */
+		if (regexec(tmpcolor->end, &fileptr->data[startx], 1,
+			&endmatch, (startx == 0) ? 0 : REG_NOTBOL) == 0) {
+		    startx += endmatch.rm_eo;
+		    /* Step ahead when both start and end are mere anchors. */
+		    if (startmatch.rm_so == startmatch.rm_eo &&
+				endmatch.rm_so == endmatch.rm_eo)
+			startx += 1;
+		    fileptr->multidata[tmpcolor->id] = CSTARTENDHERE;
+#ifdef DEBUG
+		    fprintf(stderr, "end found on this line\n");
+#endif
+		    continue;
+		}
+
+		/* Nice, we didn't find the end regex on this line.  Let's start looking for it. */
+		for (endptr = fileptr->next; endptr != NULL; endptr = endptr->next) {
+#ifdef DEBUG
+		    fprintf(stderr, "\nadvancing to line %ld to find end... ", (long)endptr->lineno);
+#endif
+		    /* Check for interrupting keyboard input again. */
+		    if (key_was_pressed())
+			goto precalc_cleanup;
+
+		    if (regexec(tmpcolor->end, endptr->data, 1, &endmatch, 0) == 0)
+			break;
+		}
+
+		if (endptr == NULL) {
+#ifdef DEBUG
+		    fprintf(stderr, "no end found, breaking out\n");
+#endif
+		    break;
+		}
+#ifdef DEBUG
+		fprintf(stderr, "end found\n");
+#endif
+		/* We found it, we found it, la la la la la.  Mark all
+		 * the lines in between and the end properly. */
+		fileptr->multidata[tmpcolor->id] = CENDAFTER;
+#ifdef DEBUG
+		fprintf(stderr, "marking line %ld as CENDAFTER\n", (long)fileptr->lineno);
+#endif
+		for (fileptr = fileptr->next; fileptr != endptr; fileptr = fileptr->next) {
+		    alloc_multidata_if_needed(fileptr);
+		    fileptr->multidata[tmpcolor->id] = CWHOLELINE;
+#ifdef DEBUG
+		    fprintf(stderr, "marking intermediary line %ld as CWHOLELINE\n", (long)fileptr->lineno);
+#endif
+		}
+
+		alloc_multidata_if_needed(endptr);
+		fileptr->multidata[tmpcolor->id] = CBEGINBEFORE;
+#ifdef DEBUG
+		fprintf(stderr, "marking line %ld as CBEGINBEFORE\n", (long)fileptr->lineno);
+#endif
+		/* Skip to the end point of the match. */
+		startx = endmatch.rm_eo;
+#ifdef DEBUG
+		fprintf(stderr, "jumping to line %ld pos %lu to continue\n", (long)fileptr->lineno, (unsigned long)startx);
+#endif
+	    }
+
+	    if (nostart && startx == 0) {
+#ifdef DEBUG
+		fprintf(stderr, "no match\n");
+#endif
+		fileptr->multidata[tmpcolor->id] = CNONE;
+		continue;
+	    }
+	}
+    }
+precalc_cleanup:
+    nodelay(edit, FALSE);
+}
+
 #endif /* !DISABLE_COLOR */
