@@ -1,4 +1,4 @@
-/* $Id: cut.c 5614 2016-02-06 12:12:08Z bens $ */
+/* $Id: cut.c 5671 2016-02-23 12:07:58Z bens $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -134,9 +134,8 @@ void do_cut_text(
 
     assert(openfile->current != NULL && openfile->current->data != NULL);
 
-    /* If keep_cutbuffer is FALSE and the cutbuffer isn't empty, blow
-     * away the text in the cutbuffer. */
-    if (!keep_cutbuffer && cutbuffer != NULL) {
+    /* Empty the cutbuffer when a chain of cuts is broken. */
+    if (!keep_cutbuffer) {
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
 #ifdef DEBUG
@@ -238,6 +237,11 @@ void do_copy_text(void)
     static struct filestruct *next_contiguous_line = NULL;
     bool mark_set = openfile->mark_set;
 
+    /* Remember the current view port and cursor position. */
+    ssize_t is_edittop_lineno = openfile->edittop->lineno;
+    ssize_t is_current_lineno = openfile->current->lineno;
+    size_t is_current_x = openfile->current_x;
+
     if (mark_set || openfile->current != next_contiguous_line)
 	cutbuffer_reset();
 
@@ -245,6 +249,13 @@ void do_copy_text(void)
 
     /* If the mark was set, blow away the cutbuffer on the next copy. */
     next_contiguous_line = (mark_set ? NULL : openfile->current);
+
+    if (mark_set) {
+	/* Restore the view port and cursor position. */
+	openfile->edittop = fsfromline(is_edittop_lineno);
+	openfile->current = fsfromline(is_current_lineno);
+	openfile->current_x = is_current_x;
+    }
 }
 
 /* Cut from the current cursor position to the end of the file. */
