@@ -1,4 +1,4 @@
-/* $Id: files.c 5629 2016-02-10 20:29:23Z bens $ */
+/* $Id: files.c 5678 2016-02-25 18:58:17Z bens $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -71,17 +71,25 @@ bool has_valid_path(const char *filename)
  * called from open_buffer(). */
 void make_new_buffer(void)
 {
-    /* If there are no entries in openfile, make the first one and
-     * move to it. */
     if (openfile == NULL) {
 	openfile = make_new_opennode();
-	splice_opennode(openfile, openfile, openfile);
-    /* Otherwise, make a new entry for openfile, splice it in after
-     * the current entry, and move to it. */
+
+	/* Make the first open file the only element in a circular list. */
+	openfile->prev = openfile;
+	openfile->next = openfile;
     } else {
-	splice_opennode(openfile, make_new_opennode(), openfile->next);
-	openfile = openfile->next;
-	/* More than one file open, show Close in help lines. */
+	openfilestruct *newnode = make_new_opennode();
+
+	/* Add the new open file after the current one in the list. */
+	newnode->prev = openfile;
+	newnode->next = openfile->next;
+	openfile->next->prev = newnode;
+	openfile->next = newnode;
+
+	/* Make the new file the current one. */
+	openfile = newnode;
+
+	/* There is more than one file open: show Close in help lines. */
 	exitfunc->desc = close_tag;
     }
 
@@ -128,10 +136,6 @@ void initialize_buffer_text(void)
     openfile->edittop = openfile->fileage;
     openfile->current = openfile->fileage;
 
-#ifndef DISABLE_COLOR
-    openfile->fileage->multidata = NULL;
-#endif
-
     openfile->totsize = 0;
 }
 
@@ -871,6 +875,7 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 	/* If the file we got doesn't end in a newline, tack its last
 	 * line onto the beginning of the line at current. */
 	if (len > 0) {
+	    filestruct *dropline = fileptr;
 	    size_t current_len = strlen(openfile->current->data);
 
 	    /* Adjust the current x-coordinate to compensate for the
@@ -896,11 +901,10 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 	    if (fileptr == openfile->filebot)
 		openfile->filebot = openfile->current;
 
-	    /* Move fileptr back one line and blow away the old fileptr,
-	     * since its text has been saved. */
+	    /* Step back one line, and blow away the unterminated line,
+	     * since its text has been copied into current. */
 	    fileptr = fileptr->prev;
-	    if (fileptr != NULL)
-		free(fileptr->next);
+	    delete_node(dropline);
 	}
 
 	/* Attach the line at current after the line at fileptr. */
@@ -1413,14 +1417,13 @@ char *get_full_path(const char *origpath)
     /* Get the current directory.  If it doesn't exist, back up and try
      * again until we get a directory that does, and use that as the
      * current directory. */
-    d_here = charalloc(PATH_MAX + 1);
-    d_here = getcwd(d_here, PATH_MAX + 1);
+    d_here = getcwd(NULL, PATH_MAX + 1);
 
     while (d_here == NULL) {
 	if (chdir("..") == -1)
 	    break;
 
-	d_here = getcwd(d_here, PATH_MAX + 1);
+	d_here = getcwd(NULL, PATH_MAX + 1);
     }
 
     /* If we succeeded, canonicalize it in d_here. */
@@ -1481,8 +1484,7 @@ char *get_full_path(const char *origpath)
 	    free(d_there);
 
 	    /* Get the full path. */
-	    d_there = charalloc(PATH_MAX + 1);
-	    d_there = getcwd(d_there, PATH_MAX + 1);
+	    d_there = getcwd(NULL, PATH_MAX + 1);
 
 	    /* If we succeeded, canonicalize it in d_there. */
 	    if (d_there != NULL) {
@@ -2556,8 +2558,8 @@ char *real_dir_from_tilde(const char *buf)
 
 	    do {
 		userdata = getpwent();
-	    } while (userdata != NULL && strcmp(userdata->pw_name,
-		tilde_dir + 1) != 0);
+	    } while (userdata != NULL &&
+			strcmp(userdata->pw_name, tilde_dir + 1) != 0);
 	    endpwent();
 	    if (userdata != NULL)
 		tilde_dir = mallocstrcpy(tilde_dir, userdata->pw_dir);
@@ -2605,6 +2607,7 @@ void free_chararray(char **array, size_t len)
 
     for (; len > 0; len--)
 	free(array[len - 1]);
+
     free(array);
 }
 #endif
@@ -2671,9 +2674,8 @@ char **username_tab_completion(const char *buf, size_t *num_matches,
 #endif
 
 	    matches = (char **)nrealloc(matches, (*num_matches + 1) *
-		sizeof(char *));
-	    matches[*num_matches] =
-		charalloc(strlen(userdata->pw_name) + 2);
+					sizeof(char *));
+	    matches[*num_matches] = charalloc(strlen(userdata->pw_name) + 2);
 	    sprintf(matches[*num_matches], "~%s", userdata->pw_name);
 	    ++(*num_matches);
 	}
@@ -2699,8 +2701,7 @@ char **cwd_tab_completion(const char *buf, bool allow_files, size_t
     *num_matches = 0;
     null_at(&dirname, buf_len);
 
-    /* Okie, if there's a / in the buffer, strip out the directory
-     * part. */
+    /* Okie, if there's a / in the buffer, strip out the directory part. */
     filename = strrchr(dirname, '/');
     if (filename != NULL) {
 	char *tmpdirname = filename + 1;
@@ -2742,8 +2743,7 @@ char **cwd_tab_completion(const char *buf, bool allow_files, size_t
 	    /* Cool, found a match.  Add it to the list.  This makes a
 	     * lot more sense to me (Chris) this way... */
 
-	    char *tmp = charalloc(strlen(dirname) +
-		strlen(nextdir->d_name) + 1);
+	    char *tmp = charalloc(strlen(dirname) + strlen(nextdir->d_name) + 1);
 	    sprintf(tmp, "%s%s", dirname, nextdir->d_name);
 
 #ifndef DISABLE_OPERATINGDIR
@@ -2764,7 +2764,7 @@ char **cwd_tab_completion(const char *buf, bool allow_files, size_t
 		continue;
 
 	    matches = (char **)nrealloc(matches, (*num_matches + 1) *
-		sizeof(char *));
+					sizeof(char *));
 	    matches[*num_matches] = mallocstrcpy(NULL, nextdir->d_name);
 	    ++(*num_matches);
 	}
@@ -2797,8 +2797,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	const char *bob = strchr(buf, '/');
 
 	if (bob == NULL || bob >= buf + *place)
-	    matches = username_tab_completion(buf, &num_matches,
-		*place);
+	    matches = username_tab_completion(buf, &num_matches, *place);
     }
 
     /* Match against files relative to the current working directory. */
@@ -2813,8 +2812,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	size_t match, common_len = 0;
 	char *mzero;
 	const char *lastslash = revstrstr(buf, "/", buf + *place);
-	size_t lastslash_len = (lastslash == NULL) ? 0 :
-		lastslash - buf + 1;
+	size_t lastslash_len = (lastslash == NULL) ? 0 : lastslash - buf + 1;
 	char *match1_mb = charalloc(mb_cur_max() + 1);
 	char *match2_mb = charalloc(mb_cur_max() + 1);
 	int match1_mb_len, match2_mb_len;
@@ -2836,7 +2834,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	    if (match < num_matches || matches[0][common_len] == '\0')
 		break;
 
-	    common_len += parse_mbchar(buf + common_len, NULL, NULL);
+	    common_len += parse_mbchar(matches[0] + common_len, NULL, NULL);
 	}
 
 	free(match1_mb);
@@ -2861,20 +2859,17 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	if (num_matches > 1 && (common_len != *place || !*lastwastab))
 	    beep();
 
-	/* If there is more of a match to display on the statusbar, show
-	 * it.  We reset lastwastab to FALSE: it requires pressing Tab
-	 * twice in succession with no statusbar changes to see a match
-	 * list. */
+	/* If the matches have something in common, show that part. */
 	if (common_len != *place) {
-	    *lastwastab = FALSE;
 	    buf = charealloc(buf, common_len + buf_len - *place + 1);
-	    charmove(buf + common_len, buf + *place, buf_len -
-		*place + 1);
+	    charmove(buf + common_len, buf + *place, buf_len - *place + 1);
 	    strncpy(buf, mzero, common_len);
 	    *place = common_len;
-	} else if (!*lastwastab || num_matches < 2)
+	}
+
+	if (!*lastwastab)
 	    *lastwastab = TRUE;
-	else {
+	else if (num_matches > 1) {
 	    int longest_name = 0, ncols, editline = 0;
 
 	    /* Now we show a list of the available choices. */
@@ -2907,7 +2902,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	    blank_edit();
 	    wmove(edit, 0, 0);
 
-	    /* Disable el cursor. */
+	    /* Don't show a cursor in the list of completions. */
 	    curs_set(0);
 
 	    for (match = 0; match < num_matches; match++) {
@@ -2923,8 +2918,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 		    break;
 		}
 
-		disp = display_string(matches[match], 0, longest_name,
-			FALSE);
+		disp = display_string(matches[match], 0, longest_name, FALSE);
 		waddstr(edit, disp);
 		free(disp);
 
@@ -2946,9 +2940,6 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
     if (!*list)
 	refresh_func();
 
-    /* Enable el cursor. */
-    curs_set(1);
-
     return buf;
 }
 #endif /* !DISABLE_TABCOMP */
@@ -2969,8 +2960,7 @@ const char *tail(const char *foo)
 
 #ifndef DISABLE_HISTORIES
 /* Return the constructed dirfile path, or NULL if we can't find the home
- * directory.  The string is dynamically allocated, and should be
- * freed. */
+ * directory.  The string is dynamically allocated, and should be freed. */
 char *construct_filename(const char *str)
 {
     char *newstr = NULL;
@@ -3177,7 +3167,9 @@ void save_poshistory(void)
 	chmod(poshist, S_IRUSR | S_IWUSR);
 
 	for (posptr = position_history; posptr != NULL; posptr = posptr->next) {
-	    statusstr = charalloc(strlen(posptr->filename) + 2 * sizeof(ssize_t) + 4);
+	    /* Assume 20 decimal positions each for line and column number,
+	     * plus two spaces, plus the line feed, plus the null byte. */
+	    statusstr = charalloc(strlen(posptr->filename) + 44);
 	    sprintf(statusstr, "%s %ld %ld\n", posptr->filename, (long)posptr->lineno,
 			(long)posptr->xno);
 	    if (fwrite(statusstr, sizeof(char), strlen(statusstr), hist) < strlen(statusstr))
