@@ -1,4 +1,4 @@
-/* $Id: global.c 5607 2016-02-05 12:27:54Z bens $ */
+/* $Id: global.c 5659 2016-02-22 09:34:33Z bens $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -1623,6 +1623,17 @@ int strtomenu(const char *input)
 
 
 #ifdef DEBUG
+#ifndef DISABLE_COLOR
+void free_list_item(regexlisttype *dropit)
+{
+    free(dropit->ext_regex);
+    if (dropit->ext != NULL)
+	regfree(dropit->ext);
+    free(dropit->ext);
+    free(dropit);
+}
+#endif
+
 /* This function is used to gracefully return all the memory we've used.
  * It should be called just before calling exit().  Practically, the
  * only effect is to cause a segmentation fault if the various data
@@ -1654,15 +1665,16 @@ void thanks_for_all_the_fish(void)
 #ifndef DISABLE_SPELLER
     free(alt_speller);
 #endif
-    if (cutbuffer != NULL)
-	free_filestruct(cutbuffer);
+    free_filestruct(cutbuffer);
 #ifndef DISABLE_JUSTIFY
-    if (jusbuffer != NULL)
-	free_filestruct(jusbuffer);
+    free_filestruct(jusbuffer);
 #endif
     /* Free the memory associated with each open file buffer. */
-    if (openfile != NULL)
-	free_openfilestruct(openfile);
+    while (openfile != openfile->next) {
+	openfile = openfile->next;
+	delete_opennode(openfile->prev);
+    }
+    delete_opennode(openfile);
 #ifndef DISABLE_COLOR
     free(syntaxstr);
     while (syntaxes != NULL) {
@@ -1671,47 +1683,32 @@ void thanks_for_all_the_fish(void)
 	free(syntaxes->desc);
 	free(syntaxes->linter);
 	free(syntaxes->formatter);
+
 	while (syntaxes->extensions != NULL) {
 	    regexlisttype *bob = syntaxes->extensions;
 	    syntaxes->extensions = bob->next;
-	    free(bob->ext_regex);
-	    if (bob->ext != NULL) {
-		regfree(bob->ext);
-		free(bob->ext);
-	    }
-	    free(bob);
+	    free_list_item(bob);
 	}
 	while (syntaxes->headers != NULL) {
 	    regexlisttype *bob = syntaxes->headers;
 	    syntaxes->headers = bob->next;
-	    free(bob->ext_regex);
-	    if (bob->ext != NULL) {
-		regfree(bob->ext);
-		free(bob->ext);
-	    }
-	    free(bob);
+	    free_list_item(bob);
 	}
 	while (syntaxes->magics != NULL) {
 	    regexlisttype *bob = syntaxes->magics;
 	    syntaxes->magics = bob->next;
-	    free(bob->ext_regex);
-	    if (bob->ext != NULL) {
-		regfree(bob->ext);
-		free(bob->ext);
-	    }
-	    free(bob);
+	    free_list_item(bob);
 	}
+
 	while (syntaxes->color != NULL) {
 	    colortype *bob = syntaxes->color;
-
 	    syntaxes->color = bob->next;
 	    free(bob->start_regex);
 	    if (bob->start != NULL) {
 		regfree(bob->start);
 		free(bob->start);
 	    }
-	    if (bob->end_regex != NULL)
-		free(bob->end_regex);
+	    free(bob->end_regex);
 	    if (bob->end != NULL) {
 		regfree(bob->end);
 		free(bob->end);
@@ -1724,17 +1721,16 @@ void thanks_for_all_the_fish(void)
 #endif /* !DISABLE_COLOR */
 #ifndef DISABLE_HISTORIES
     /* Free the search and replace history lists. */
-    if (searchage != NULL)
-	free_filestruct(searchage);
-    if (replaceage != NULL)
-	free_filestruct(replaceage);
+    free_filestruct(searchage);
+    free_filestruct(replaceage);
 #endif
-    /* Free the functions and shortcuts lists. */
+    /* Free the list of functions. */
     while (allfuncs != NULL) {
 	subnfunc *f = allfuncs;
 	allfuncs = allfuncs->next;
 	free(f);
     }
+    /* Free the list of shortcuts. */
     while (sclist != NULL) {
 	sc *s = sclist;
 	sclist = sclist->next;
@@ -1744,5 +1740,4 @@ void thanks_for_all_the_fish(void)
     free(homedir);
 #endif
 }
-
 #endif /* DEBUG */
