@@ -1,4 +1,4 @@
-/* $Id: nano.c 5628 2016-02-10 20:16:50Z bens $ */
+/* $Id: nano.c 5678 2016-02-25 18:58:17Z bens $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -163,10 +163,11 @@ filestruct *copy_filestruct(const filestruct *src)
     return head;
 }
 
-/* Free a filestruct. */
+/* Free a whole linked list of filestructs. */
 void free_filestruct(filestruct *src)
 {
-    assert(src != NULL);
+    if (src == NULL)
+	return;
 
     while (src->next != NULL) {
 	src = src->next;
@@ -364,8 +365,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	    *file_bot = openfile->filebot;
 	}
 
-	openfile->fileage->next = NULL;
-	free_filestruct(openfile->fileage);
+	delete_node(openfile->fileage);
 
 	/* Renumber starting with the line after the original
 	 * file_bot. */
@@ -525,20 +525,6 @@ openfilestruct *make_new_opennode(void)
     return (openfilestruct *)nmalloc(sizeof(openfilestruct));
 }
 
-/* Splice a node into an existing openfilestruct. */
-void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
-	openfilestruct *end)
-{
-    assert(newnode != NULL && begin != NULL);
-
-    newnode->next = end;
-    newnode->prev = begin;
-    begin->next = newnode;
-
-    if (end != NULL)
-	end->prev = newnode;
-}
-
 /* Unlink a node from the rest of the openfilestruct, and delete it. */
 void unlink_opennode(openfilestruct *fileptr)
 {
@@ -551,7 +537,7 @@ void unlink_opennode(openfilestruct *fileptr)
     delete_opennode(fileptr);
 }
 
-/* Delete a node from the openfilestruct. */
+/* Free all the memory in the given open-file node. */
 void delete_opennode(openfilestruct *fileptr)
 {
     assert(fileptr != NULL && fileptr->filename != NULL && fileptr->fileage != NULL);
@@ -567,22 +553,6 @@ void delete_opennode(openfilestruct *fileptr)
     free(fileptr);
 }
 
-#ifdef DEBUG
-/* Deallocate all memory associated with this and later files, including
- * the lines of text. */
-void free_openfilestruct(openfilestruct *src)
-{
-    assert(src != NULL);
-
-    while (src != src->next) {
-	src = src->next;
-	delete_opennode(src->prev);
-    }
-
-    delete_opennode(src);
-}
-#endif
-
 /* Display a warning about a key disabled in view mode. */
 void print_view_warning(void)
 {
@@ -1099,8 +1069,6 @@ int no_help(void)
  * name with the TEMP_FILE flag set, just before the filename prompt. */
 void no_current_file_name_warning(void)
 {
-    curs_set(0);
-
     /* Warn that the current file has no name. */
     statusbar(_("No file name"));
     beep();
@@ -2643,10 +2611,6 @@ int main(int argc, char **argv)
 	UNSET(MULTIBUFFER);
 #endif
 
-#ifdef DEBUG
-    fprintf(stderr, "Main: top and bottom win\n");
-#endif
-
     /* If a starting position was given on the command line, go there. */
     if (startline > 0 || startcol > 0)
 	do_gotolinecolumn(startline, startcol, FALSE, FALSE);
@@ -2659,6 +2623,10 @@ int main(int argc, char **argv)
     }
 #endif
 
+#ifdef DEBUG
+    fprintf(stderr, "Main: bottom win, top win and edit win\n");
+#endif
+
     display_main_list();
 
     display_buffer();
@@ -2675,6 +2643,10 @@ int main(int argc, char **argv)
 	    do_cursorpos(TRUE);
 
 	currmenu = MMAIN;
+	focusing = FALSE;
+
+	/* Turn the cursor on when waiting for input. */
+	curs_set(1);
 
 	/* Read in and interpret characters. */
 	do_input(TRUE);
