@@ -1,4 +1,4 @@
-/* $Id: nano.h 5608 2016-02-06 10:59:31Z bens $ */
+/* $Id: nano.h 5679 2016-02-25 21:04:45Z astyanax $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -517,7 +517,8 @@ enum
     POS_HISTORY,
     LOCKING,
     NOREAD_MODE,
-    MAKE_IT_UNIX
+    MAKE_IT_UNIX,
+    JUSTIFY_TRIM
 };
 
 /* Flags for the menus in which a given function should be present. */
