@@ -1,4 +1,4 @@
-/* $Id: prompt.c 5620 2016-02-07 13:41:46Z bens $ */
+/* $Id: prompt.c 5670 2016-02-23 08:31:57Z bens $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -122,18 +122,7 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 	if (kbinput != NULL) {
 	    /* Display all the characters in the input buffer at
 	     * once, filtering out control characters. */
-	    char *output = charalloc(kbinput_len + 1);
-	    size_t i;
-	    bool got_enter;
-		/* Whether we got the Enter key. */
-
-	    for (i = 0; i < kbinput_len; i++)
-		output[i] = (char)kbinput[i];
-	    output[i] = '\0';
-
-	    do_statusbar_output(output, kbinput_len, &got_enter, FALSE);
-
-	    free(output);
+	    do_statusbar_output(kbinput, kbinput_len, TRUE, NULL);
 
 	    /* Empty the input buffer. */
 	    kbinput_len = 0;
@@ -174,8 +163,8 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 		 * prompt, disable verbatim input. */
 		if (!ISSET(RESTRICTED) || currmenu != MWRITEFILE ||
 			openfile->filename[0] == '\0') {
-		    bool got_enter;
-		    /* Whether we got the Enter key. */
+		    bool got_enter = FALSE;
+			/* Whether we got the Enter key. */
 
 		    do_statusbar_verbatim_input(&got_enter);
 
@@ -251,66 +240,61 @@ int do_statusbar_mouse(void)
 }
 #endif
 
-/* The user typed output_len multibyte characters.  Add them to the
- * statusbar prompt, setting got_enter to TRUE if we get a newline, and
- * filtering out all ASCII control characters if allow_cntrls is
- * TRUE. */
-void do_statusbar_output(char *output, size_t output_len, bool
-	*got_enter, bool allow_cntrls)
+/* The user typed input_len multibyte characters.  Add them to the
+ * statusbar prompt, setting got_enter to TRUE if we get a newline,
+ * and filtering out ASCII control characters if filtering is TRUE. */
+void do_statusbar_output(int *the_input, size_t input_len,
+	bool filtering, bool *got_enter)
 {
-    size_t answer_len, i = 0;
+    char *output = charalloc(input_len + 1);
     char *char_buf = charalloc(mb_cur_max());
-    int char_buf_len;
+    int i, char_len;
 
     assert(answer != NULL);
 
-    answer_len = strlen(answer);
-    *got_enter = FALSE;
+    /* Copy the typed stuff so it can be treated. */
+    for (i = 0; i < input_len; i++)
+	output[i] = (char)the_input[i];
+    output[i] = '\0';
 
-    while (i < output_len) {
-	/* If allow_cntrls is TRUE, convert nulls and newlines
-	 * properly. */
-	if (allow_cntrls) {
-	    /* Null to newline, if needed. */
+    i = 0;
+
+    while (i < input_len) {
+	/* When not filtering, convert nulls and stop at a newline. */
+	if (!filtering) {
 	    if (output[i] == '\0')
 		output[i] = '\n';
-	    /* Newline to Enter, if needed. */
 	    else if (output[i] == '\n') {
-		/* Set got_enter to TRUE to indicate that we got the
-		 * Enter key, put back the rest of the characters in
-		 * output so that they can be parsed and output again,
-		 * and get out. */
+		/* Put back the rest of the characters for reparsing,
+		 * indicate that we got the Enter key and get out. */
+		unparse_kbinput(output + i, input_len - i);
 		*got_enter = TRUE;
-		unparse_kbinput(output + i, output_len - i);
 		return;
 	    }
 	}
 
 	/* Interpret the next multibyte character. */
-	char_buf_len = parse_mbchar(output + i, char_buf, NULL);
+	char_len = parse_mbchar(output + i, char_buf, NULL);
 
-	i += char_buf_len;
+	i += char_len;
 
-	/* If allow_cntrls is FALSE, filter out an ASCII control
-	 * character. */
-	if (!allow_cntrls && is_ascii_cntrl_char(*(output + i -
-		char_buf_len)))
+	/* When filtering, skip any ASCII control character. */
+	if (filtering && is_ascii_cntrl_char(*(output + i - char_len)))
 	    continue;
 
-	/* More dangerousness fun. =) */
-	answer = charealloc(answer, answer_len + (char_buf_len * 2));
+	assert(statusbar_x <= strlen(answer));
 
-	assert(statusbar_x <= answer_len);
+	/* Insert the typed character into the existing answer string. */
+	answer = charealloc(answer, strlen(answer) + char_len + 1);
+	charmove(answer + statusbar_x + char_len, answer + statusbar_x,
+				strlen(answer) - statusbar_x + 1);
+	strncpy(answer + statusbar_x, char_buf, char_len);
 
-	charmove(answer + statusbar_x + char_buf_len, answer + statusbar_x,
-			answer_len - statusbar_x + char_buf_len);
-	strncpy(answer + statusbar_x, char_buf, char_buf_len);
-	answer_len += char_buf_len;
-
-	statusbar_x += char_buf_len;
+	statusbar_x += char_len;
     }
 
     free(char_buf);
+    free(output);
 
     statusbar_pww = statusbar_xplustabs();
 
@@ -364,15 +348,13 @@ void do_statusbar_delete(void)
     statusbar_pww = statusbar_xplustabs();
 
     if (answer[statusbar_x] != '\0') {
-	int char_buf_len = parse_mbchar(answer + statusbar_x, NULL, NULL);
-	size_t line_len = strlen(answer + statusbar_x);
+	int char_len = parse_mbchar(answer + statusbar_x, NULL, NULL);
 
 	assert(statusbar_x < strlen(answer));
 
-	charmove(answer + statusbar_x, answer + statusbar_x + char_buf_len,
-			strlen(answer) - statusbar_x - char_buf_len + 1);
-
-	null_at(&answer, statusbar_x + line_len - char_buf_len);
+	charmove(answer + statusbar_x, answer + statusbar_x + char_len,
+			strlen(answer) - statusbar_x - char_len + 1);
+	align(&answer);
 
 	update_the_statusbar();
     }
@@ -453,25 +435,14 @@ void do_statusbar_prev_word(void)
 void do_statusbar_verbatim_input(bool *got_enter)
 {
     int *kbinput;
-    size_t kbinput_len, i;
-    char *output;
-
-    *got_enter = FALSE;
+    size_t kbinput_len;
 
     /* Read in all the verbatim characters. */
     kbinput = get_verbatim_kbinput(bottomwin, &kbinput_len);
 
     /* Display all the verbatim characters at once, not filtering out
      * control characters. */
-    output = charalloc(kbinput_len + 1);
-
-    for (i = 0; i < kbinput_len; i++)
-	output[i] = (char)kbinput[i];
-    output[i] = '\0';
-
-    do_statusbar_output(output, kbinput_len, got_enter, TRUE);
-
-    free(output);
+    do_statusbar_output(kbinput, kbinput_len, FALSE, got_enter);
 }
 
 /* Return the placewewant associated with statusbar_x, i.e. the
@@ -610,6 +581,9 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
      * this case, disable all keys that would change the text if the
      * filename isn't blank and we're at the "Write File" prompt. */
     while (TRUE) {
+	/* Ensure the cursor is on when waiting for input. */
+	curs_set(1);
+
 	kbinput = do_statusbar_input(&ran_func, &finished, refresh_func);
 	assert(statusbar_x <= strlen(answer));
 
@@ -636,9 +610,8 @@ functionptrtype get_prompt_string(int *actual, bool allow_tabs,
 		    complete_len = strlen(answer);
 
 		if (complete_len > 0) {
-		    answer = mallocstrcpy(answer,
-				get_history_completion(history_list,
-					answer, complete_len));
+		    answer = get_history_completion(history_list,
+					answer, complete_len);
 		    statusbar_x = strlen(answer);
 		}
 	    } else
