@@ -1,4 +1,4 @@
-/* $Id: proto.h 5612 2016-02-06 11:44:33Z bens $ */
+/* $Id: proto.h 5678 2016-02-25 18:58:17Z bens $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -164,7 +164,7 @@ void do_last_file(void);
 char *striponedir(const char *path);
 #endif
 
-/* All functions in chars.c. */
+/* Most functions in chars.c. */
 #ifdef ENABLE_UTF8
 void utf8_init(void);
 bool using_utf8(void);
@@ -189,7 +189,7 @@ bool is_punct_mbchar(const char *c);
 bool is_word_mbchar(const char *c, bool allow_punct);
 char control_rep(char c);
 #ifdef ENABLE_UTF8
-wchar_t control_wrep(wchar_t c);
+wchar_t control_wrep(wchar_t wc);
 #endif
 char *control_mbrep(const char *c, char *crep, int *crep_len);
 char *mbrep(const char *c, char *crep, int *crep_len);
@@ -247,7 +247,7 @@ bool is_valid_unicode(wchar_t wc);
 bool is_valid_mbstring(const char *s);
 #endif
 
-/* All functions in color.c. */
+/* Most functions in color.c. */
 #ifndef DISABLE_COLOR
 void set_colorpairs(void);
 void color_init(void);
@@ -359,10 +359,10 @@ int check_poshistory(const char *file, ssize_t *line, ssize_t *column);
 
 /* Some functions in global.c. */
 size_t length_of_list(int menu);
-key_type strtokeytype(const char *str);
 const sc *first_sc_for(int menu, void (*func)(void));
 int sc_seq_or(void (*func)(void), int defaultval);
 functionptrtype func_from_key(int *kbinput);
+key_type strtokeytype(const char *str);
 void assign_keyinfo(sc *s);
 void print_sclist(void);
 void shortcut_init(void);
@@ -399,10 +399,10 @@ void do_para_end(bool allow_update);
 void do_para_end_void(void);
 #endif
 #ifndef NANO_TINY
-bool do_next_word(bool allow_punct, bool allow_update);
-void do_next_word_void(void);
 void do_prev_word(bool allow_punct, bool allow_update);
 void do_prev_word_void(void);
+bool do_next_word(bool allow_punct, bool allow_update);
+void do_next_word_void(void);
 #endif
 void do_home(void);
 void do_end(void);
@@ -447,13 +447,8 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	filestruct *top, size_t top_x, filestruct *bot, size_t bot_x);
 void copy_from_filestruct(filestruct *somebuffer);
 openfilestruct *make_new_opennode(void);
-void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
-	openfilestruct *end);
 void unlink_opennode(openfilestruct *fileptr);
 void delete_opennode(openfilestruct *fileptr);
-#ifdef DEBUG
-void free_openfilestruct(openfilestruct *src);
-#endif
 void print_view_warning(void);
 void show_restricted_warning(void);
 #ifdef DISABLE_HELP
@@ -517,8 +512,8 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 #ifndef DISABLE_MOUSE
 int do_statusbar_mouse(void);
 #endif
-void do_statusbar_output(char *output, size_t output_len, bool
-	*got_enter, bool allow_cntrls);
+void do_statusbar_output(int *the_input, size_t input_len,
+	bool filtering, bool *got_enter);
 void do_statusbar_home(void);
 void do_statusbar_end(void);
 void do_statusbar_left(void);
@@ -527,8 +522,8 @@ void do_statusbar_backspace(void);
 void do_statusbar_delete(void);
 void do_statusbar_cut_text(void);
 #ifndef NANO_TINY
-void do_statusbar_next_word(void);
 void do_statusbar_prev_word(void);
+void do_statusbar_next_word(void);
 #endif
 void do_statusbar_verbatim_input(bool *got_enter);
 size_t statusbar_xplustabs(void);
@@ -536,7 +531,6 @@ size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reset_statusbar_cursor(void);
 void update_the_statusbar(void);
 void update_bar_if_needed(void);
-void total_statusbar_refresh(void (*refresh_func)(void));
 functionptrtype get_prompt_string(int *value, bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
@@ -556,7 +550,6 @@ int do_prompt(bool allow_tabs,
 	filestruct **history_list,
 #endif
 	void (*refresh_func)(void), const char *msg, ...);
-void do_prompt_abort(void);
 int do_yesno_prompt(bool all, const char *msg);
 
 /* Most functions in rcfile.c. */
@@ -625,7 +618,7 @@ void do_gotolinecolumn_void(void);
 bool find_bracket_match(bool reverse, const char *bracket_set);
 void do_find_bracket(void);
 #ifndef DISABLE_TABCOMP
-char *get_history_completion(filestruct **h, const char *s, size_t len);
+char *get_history_completion(filestruct **h, char *s, size_t len);
 #endif
 #endif
 #ifndef DISABLE_HISTORIES
