@@ -1,4 +1,4 @@
-/* $Id: search.c 5593 2016-01-26 09:16:09Z bens $ */
+/* $Id: search.c 5653 2016-02-20 12:16:43Z bens $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -190,8 +190,6 @@ int search_init(bool replacing, bool use_answer)
 #endif
 	_(" (to replace)") : "", buf);
 
-    fflush(stderr);
-
     /* Release buf now that we don't need it anymore. */
     free(buf);
 
@@ -708,8 +706,10 @@ ssize_t do_replace_loop(
 		xpt, strnlenpt(openfile->current->data,
 		openfile->current_x + match_len) - xpt, FALSE);
 
+	    /* Refresh the edit window, scrolling it if necessary. */
 	    edit_refresh();
 
+	    /* Don't show cursor, to not distract from highlighted match. */
 	    curs_set(0);
 
 	    do_replace_highlight(TRUE, exp_word);
@@ -721,8 +721,6 @@ ssize_t do_replace_loop(
 
 	    free(exp_word);
 
-	    curs_set(1);
-
 	    if (i == -1) {	/* We canceled the replace. */
 		if (canceled != NULL)
 		    *canceled = TRUE;
@@ -1353,7 +1351,7 @@ void get_history_older_void(void)
  * looking at only the first len characters of s, and return that
  * string.  If there isn't one, or if len is 0, don't move h and return
  * s. */
-char *get_history_completion(filestruct **h, const char *s, size_t len)
+char *get_history_completion(filestruct **h, char *s, size_t len)
 {
     assert(s != NULL);
 
@@ -1382,7 +1380,7 @@ char *get_history_completion(filestruct **h, const char *s, size_t len)
 
 	if (p != NULL) {
 	    *h = p;
-	    return (*h)->data;
+	    return mallocstrcpy(s, (*h)->data);
 	}
 
 	/* Search the history list from the top to the current position
@@ -1394,7 +1392,7 @@ char *get_history_completion(filestruct **h, const char *s, size_t len)
 
 	if (p != NULL) {
 	    *h = p;
-	    return (*h)->data;
+	    return mallocstrcpy(s, (*h)->data);
 	}
     }
 
